module QuoineClient
  class BaseRequest

    include APIClientBase::Request.module
    attribute :user_secret, String
    attribute :user_id, String
    attribute :nonce, String, default: :default_nonce
    attribute :date_of_request, String, default: Time.now.httpdate
    attribute :signature, String, lazy: true, default: :default_signature

    private

    def headers
      {
        "Content-Type" => "application/json",
        "Date" => date_of_request,
        "NONCE" => nonce,
        "Authorization" => "APIAuth #{user_id}:#{signature}",
      }
    end

    def default_nonce
      SecureRandom.hex
    end

    def default_signature
      canonical_string = "application/json,,#{path},#{date_of_request},#{nonce}"

      Base64.strict_encode64(
        OpenSSL::HMAC.digest(
          OpenSSL::Digest.new("sha1"),
          user_secret,
          canonical_string,
        )
      )
    end

  end

end
